@@ -260,6 +260,7 @@ static void check_if_skip(e2fsck_t ctx)
 	long next_check;
 	int batt = is_on_batt();
 	int defer_check_on_battery;
+	time_t lastcheck;
 
 	profile_get_boolean(ctx->profile, "options",
 			    "defer_check_on_battery", 0, 1, 
@@ -271,11 +272,16 @@ static void check_if_skip(e2fsck_t ctx)
 	    cflag || swapfs)
 		return;
 	
+	lastcheck = fs->super->s_lastcheck;
+	if (lastcheck > ctx->now)
+		lastcheck -= ctx->time_fudge;
 	if ((fs->super->s_state & EXT2_ERROR_FS) ||
 	    !ext2fs_test_valid(fs))
 		reason = _(" contains a file system with errors");
 	else if ((fs->super->s_state & EXT2_VALID_FS) == 0)
 		reason = _(" was not cleanly unmounted");
+	else if (check_backup_super_block(ctx))
+		reason = _(" primary superblock features different from backup");
 	else if ((fs->super->s_max_mnt_count > 0) &&
 		 (fs->super->s_mnt_count >=
 		  (unsigned) fs->super->s_max_mnt_count)) {
@@ -285,8 +291,7 @@ static void check_if_skip(e2fsck_t ctx)
 			     (unsigned) fs->super->s_max_mnt_count*2))
 			reason = 0;
 	} else if (fs->super->s_checkinterval &&
-		   ((ctx->now - fs->super->s_lastcheck) >= 
-		    fs->super->s_checkinterval)) {
+		   ((ctx->now - lastcheck) >= fs->super->s_checkinterval)) {
 		reason = _(" has gone %u days without being checked");
 		reason_arg = (ctx->now - fs->super->s_lastcheck)/(3600*24);
 		if (batt && ((ctx->now - fs->super->s_lastcheck) < 
@@ -585,6 +590,10 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 #endif
 	char		*extended_opts = 0;
 	char		*cp;
+	int 		res;		/* result of sscanf */
+#ifdef CONFIG_JBD_DEBUG
+	char 		*jbd_debug;
+#endif
 
 	retval = e2fsck_allocate_context(&ctx);
 	if (retval)
@@ -614,7 +623,10 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 		switch (c) {
 		case 'C':
 			ctx->progress = e2fsck_update_progress;
-			ctx->progress_fd = atoi(optarg);
+			res = sscanf(optarg, "%d", &ctx->progress_fd);
+			if (res != 1)
+				goto sscanf_err;
+
 			if (!ctx->progress_fd)
 				break;
 			/* Validate the file descriptor to avoid disasters */
@@ -674,20 +686,26 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 			/* What we do by default, anyway! */
 			break;
 		case 'b':
-			ctx->use_superblock = atoi(optarg);
+			res = sscanf(optarg, "%d", &ctx->use_superblock);
+			if (res != 1)
+				goto sscanf_err;
 			ctx->flags |= E2F_FLAG_SB_SPECIFIED;
 			break;
 		case 'B':
 			ctx->blocksize = atoi(optarg);
 			break;
 		case 'I':
-			ctx->inode_buffer_blocks = atoi(optarg);
+			res = sscanf(optarg, "%d", &ctx->inode_buffer_blocks);
+			if (res != 1)
+				goto sscanf_err;
 			break;
 		case 'j':
 			ctx->journal_name = string_copy(ctx, optarg, 0);
 			break;
 		case 'P':
-			ctx->process_inode_size = atoi(optarg);
+			res = sscanf(optarg, "%d", &ctx->process_inode_size);
+			if (res != 1)
+				goto sscanf_err;
 			break;
 		case 'L':
 			replace_bad_blocks++;
@@ -830,10 +848,22 @@ static errcode_t PRS(int argc, char *argv[], e2fsck_t *ret_ctx)
 		putenv(newpath);
 	}
 #ifdef CONFIG_JBD_DEBUG
-	if (getenv("E2FSCK_JBD_DEBUG"))
-		journal_enable_debug = atoi(getenv("E2FSCK_JBD_DEBUG"));
+	jbd_debug = getenv("E2FSCK_JBD_DEBUG");
+	if (jbd_debug)
+		res = sscanf(jbd_debug, "%d", &journal_enable_debug);
+		if (res != 1) {
+			fprintf(stderr,
+			        _("E2FSCK_JBD_DEBUG \"%s\" not an integer\n\n"),
+			        jbd_debug);
+			exit (1);
+		}
 #endif
 	return 0;
+
+sscanf_err:
+	fprintf(stderr, _("\nInvalid non-numeric argument to -%c (\"%s\")\n\n"),
+	        c, optarg);
+	exit (1);
 }
 
 static const char *my_ver_string = E2FSPROGS_VERSION;
@@ -841,7 +871,7 @@ static const char *my_ver_date = E2FSPROGS_DATE;
 					
 int main (int argc, char *argv[])
 {
-	errcode_t	retval = 0;
+	errcode_t	retval = 0, orig_retval = 0;
 	int		exit_value = FSCK_OK;
 	ext2_filsys	fs = 0;
 	io_manager	io_ptr;
@@ -852,6 +882,7 @@ int main (int argc, char *argv[])
 	struct problem_context pctx;
 	int flags, run_result;
 	int journal_size;
+	int sysval, sys_page_size = 4096;
 	
 	clear_problem_context(&pctx);
 #ifdef MTRACE
@@ -940,18 +971,23 @@ restart:
 	if (!ctx->superblock && !(ctx->options & E2F_OPT_PREEN) &&
 	    !(ctx->flags & E2F_FLAG_SB_SPECIFIED) &&
 	    ((retval == EXT2_ET_BAD_MAGIC) ||
+	     (retval == EXT2_ET_CORRUPT_SUPERBLOCK) ||
 	     ((retval == 0) && ext2fs_check_desc(fs)))) {
 		if (!fs || (fs->group_desc_count > 1)) {
-			printf(_("%s trying backup blocks...\n"),
-			       retval ? _("Couldn't find ext2 superblock,") :
+			printf(_("%s: %s trying backup blocks...\n"),
+			       ctx->program_name, 
+			       retval ? _("Superblock invalid,") :
 			       _("Group descriptors look bad..."));
 			get_backup_sb(ctx, fs, ctx->filesystem_name, io_ptr);
 			if (fs)
 				ext2fs_close(fs);
+			orig_retval = retval;
 			goto restart;
 		}
 	}
 	if (retval) {
+		if (orig_retval)
+			retval = orig_retval;
 		com_err(ctx->program_name, retval, _("while trying to open %s"),
 			ctx->filesystem_name);
 		if (retval == EXT2_ET_REV_TOO_HIGH) {
@@ -1139,6 +1175,20 @@ restart:
 	    !(ctx->options & E2F_OPT_READONLY))
 		ext2fs_mark_super_dirty(fs);
 
+	/*
+	 * Calculate the number of filesystem blocks per pagesize.  If
+	 * fs->blocksize > page_size, set the number of blocks per
+	 * pagesize to 1 to avoid division by zero errors.
+	 */
+#ifdef _SC_PAGESIZE
+	sysval = sysconf(_SC_PAGESIZE);
+	if (sysval > 0)
+		sys_page_size = sysval;
+#endif /* _SC_PAGESIZE */
+	ctx->blocks_per_page = sys_page_size / fs->blocksize;
+	if (ctx->blocks_per_page == 0)
+		ctx->blocks_per_page = 1;
+
 	ehandler_init(fs->io);
 
 	if (ctx->superblock)
@@ -1204,6 +1254,7 @@ restart:
 			if (journal_size < 0) {
 				fs->super->s_feature_compat &=
 					~EXT3_FEATURE_COMPAT_HAS_JOURNAL;
+				fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
 				com_err(ctx->program_name, 0, 
 					_("Couldn't determine journal size"));
 				goto no_journal;
@@ -1243,6 +1294,10 @@ no_journal:
 	}
 	if (run_result & E2F_FLAG_ABORT)
 		fatal_error(ctx, _("aborted"));
+	if (check_backup_super_block(ctx)) {
+		fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
+		ext2fs_mark_super_dirty(fs);
+	}
 
 #ifdef MTRACE
 	mtrace_print("Cleanup");
