@@ -284,6 +284,8 @@ static int probe_fat(struct blkid_probe *probe,
 			(sector_size-1)) / sector_size;
 
 	cluster_count = sect_count - (reserved + fat_size + dir_size);
+	if (ms->ms_cluster_size == 0)
+		return 1;
 	cluster_count /= ms->ms_cluster_size;
 
 	if (cluster_count > FAT32_MAX)
@@ -846,8 +848,9 @@ static struct blkid_magic type_array[] = {
   { "vfat",      0,   0x36,  5, "MSDOS",                probe_fat },
   { "vfat",      0,   0x36,  8, "FAT16   ",             probe_fat },
   { "vfat",      0,   0x36,  8, "FAT12   ",             probe_fat },
-  { "vfat",      0,      0,  2, "\353\220",             probe_fat_nomagic },
+  { "vfat",      0,      0,  1, "\353",                 probe_fat_nomagic },
   { "vfat",      0,      0,  1, "\351",                 probe_fat_nomagic },
+  { "vfat",      0,  0x1fe,  2, "\125\252",             probe_fat_nomagic },
   { "minix",     1,   0x10,  2, "\177\023",             0 },
   { "minix",     1,   0x10,  2, "\217\023",             0 },
   { "minix",	 1,   0x10,  2, "\150\044",		0 },
@@ -898,6 +901,8 @@ static struct blkid_magic type_array[] = {
   { "ocfs2",	 4,	 0,  6,	"OCFSV2",		probe_ocfs2 },
   { "ocfs2",	 8,	 0,  6,	"OCFSV2",		probe_ocfs2 },
   { "crypt_LUKS", 0,	 0,  6,	"LUKS\xba\xbe",		probe_luks },
+  { "squashfs",	 0,	 0,  4,	"sqsh",			0 },
+  { "squashfs",	 0,	 0,  4,	"hsqs",			0 },
   {   NULL,	 0,	 0,  0, NULL,			NULL }
 };
 
