@@ -37,6 +37,7 @@
 int fsetversion (const char * name, unsigned long version)
 {
 #if HAVE_EXT2_IOCTLS
+#if !APPLE_DARWIN
 	int fd, r, ver, save_errno = 0;
 
 	fd = open (name, OPEN_FLAGS);
@@ -50,6 +51,10 @@ int fsetversion (const char * name, unsigned long version)
 	if (save_errno)
 		errno = save_errno;
 	return r;
+#else
+   int ver = (int)version;
+   return syscall(SYS_fsctl, name, EXT2_IOC_SETVERSION, &ver, 0);
+#endif
 #else /* ! HAVE_EXT2_IOCTLS */
 	extern int errno;
 	errno = EOPNOTSUPP;
