@@ -46,6 +46,7 @@ extern int optind;
 #include <unistd.h>
 #include <setjmp.h>
 #include <time.h>
+#include <limits.h>
 
 #include <sys/ioctl.h>
 #include <sys/types.h>
@@ -57,7 +58,7 @@ extern int optind;
 #include "nls-enable.h"
 
 const char * program_name = "badblocks";
-const char * done_string = N_("done                             \n");
+const char * done_string = N_("done                                \n");
 
 static int v_flag = 0;			/* verbose */
 static int w_flag = 0;			/* do r/w test: 0=no, 1=yes,
@@ -68,6 +69,7 @@ static int t_flag = 0;			/* number of test patterns */
 static int t_max = 0;			/* allocated test patterns */
 static unsigned long *t_patts = NULL;	/* test patterns */
 static int current_O_DIRECT = 0;	/* Current status of O_DIRECT flag */
+static int exclusive_ok = 0;
 
 #define T_INC 32
 
@@ -80,6 +82,14 @@ static void usage(void)
 	exit (1);
 }
 
+static void exclusive_usage(void)
+{
+	fprintf(stderr, 
+		_("%s: The -n and -w options are mutually exclusive.\n\n"), 
+		program_name);
+	exit(1);
+}
+
 static unsigned long currently_testing = 0;
 static unsigned long num_blocks = 0;
 static ext2_badblocks_list bb_list = NULL;
@@ -349,7 +359,7 @@ static unsigned int test_ro (int dev, unsigned long last_block,
 	}
 	if (v_flag) {
 	    fprintf (stderr, _("Checking blocks %lu to %lu\n"), from_count,
-		     last_block);
+		     last_block - 1);
 	}
 	if (t_flag) {
 		fputs(_("Checking for bad blocks in read-only mode\n"), stderr);
@@ -359,7 +369,7 @@ static unsigned int test_ro (int dev, unsigned long last_block,
 	flush_bufs();
 	try = blocks_at_once;
 	currently_testing = from_count;
-	num_blocks = last_block;
+	num_blocks = last_block - 1;
 	if (!t_flag && (s_flag || v_flag)) {
 		fputs(_("Checking for bad blocks (read-only test): "), stderr);
 		if (v_flag <= 1)
@@ -456,7 +466,7 @@ static unsigned int test_rw (int dev, unsigned long last_block,
 	for (pat_idx = 0; pat_idx < nr_pattern; pat_idx++) {
 		pattern_fill(buffer, pattern[pat_idx],
 			     blocks_at_once * block_size);
-		num_blocks = last_block;
+		num_blocks = last_block - 1;
 		currently_testing = from_count;
 		if (s_flag && v_flag <= 1)
 			alarm_intr(SIGALRM);
@@ -632,7 +642,7 @@ static unsigned int test_nd (int dev, unsigned long last_block,
 		save_ptr = save_base;
 		test_ptr = test_base;
 		currently_testing = from_count;
-		num_blocks = last_block;
+		num_blocks = last_block - 1;
 		if (s_flag && v_flag <= 1)
 			alarm_intr(SIGALRM);
 
@@ -792,7 +802,7 @@ static void check_mount(char *device_name)
 		exit(1);
 	}
 
-	if (mount_flags & EXT2_MF_BUSY) {
+	if ((mount_flags & EXT2_MF_BUSY) && !exclusive_ok) {
 		fprintf(stderr, _("%s is apparently in use by the system; "),
 			device_name);
 		if (force)
@@ -852,7 +862,7 @@ int main (int argc, char ** argv)
 	
 	if (argc && *argv)
 		program_name = *argv;
-	while ((c = getopt (argc, argv, "b:fi:o:svwnc:p:h:t:")) != EOF) {
+	while ((c = getopt (argc, argv, "b:fi:o:svwnc:p:h:t:X")) != EOF) {
 		switch (c) {
 		case 'b':
 			block_size = strtoul (optarg, &tmp, 0);
@@ -879,13 +889,13 @@ int main (int argc, char ** argv)
 			break;
 		case 'w':
 			if (w_flag)
-				usage();
+				exclusive_usage();
 			test_func = test_rw;
 			w_flag = 1;
 			break;
 		case 'n':
 			if (w_flag)
-				usage();
+				exclusive_usage();
 			test_func = test_nd;
 			w_flag = 2;
 			break;
@@ -938,6 +948,9 @@ int main (int argc, char ** argv)
 				t_patts[t_flag++] = pattern;
 			}
 			break;
+		case 'X':
+			exclusive_ok++;
+			break;
 		default:
 			usage();
 		}
@@ -975,24 +988,31 @@ int main (int argc, char ** argv)
 			exit(1);
 		}
 	} else {
+		errno = 0;
 		last_block = strtoul (argv[optind], &tmp, 0);
-		if (*tmp) {
+		printf("last_block = %d (%s)\n", last_block, argv[optind]);
+		if (*tmp || errno || 
+		    (last_block == ULONG_MAX && errno == ERANGE)) {
 			com_err (program_name, 0, _("invalid blocks count - %s"),
 				 argv[optind]);
 			exit (1);
 		}
+		last_block++;
 		optind++;
 	}
 	if (optind <= argc-1) {
+		errno = 0;
 		from_count = strtoul (argv[optind], &tmp, 0);
-		if (*tmp) {
+		printf("from_count = %d\n", from_count);
+		if (*tmp || errno ||
+		    (from_count == ULONG_MAX && errno == ERANGE)) {
 			com_err (program_name, 0, _("invalid starting block - %s"),
 				 argv[optind]);
 			exit (1);
 		}
 	} else from_count = 0;
 	if (from_count >= last_block) {
-	    com_err (program_name, 0, _("invalid blocks range: %lu-%lu"),
+	    com_err (program_name, 0, _("invalid starting block (%d): must be less than %lu"),
 		     (unsigned long) from_count, (unsigned long) last_block);
 	    exit (1);
 	}
