@@ -275,20 +275,17 @@ static int parse_fstab_line(char *line, struct fs_info **ret_fs)
 
 	*ret_fs = 0;
 	strip_line(line);
-	if ((cp = strchr(line, '#')))
-		*cp = 0;	/* Ignore everything after the comment char */
 	cp = line;
 
 	device = parse_word(&cp);
+	if (!device || *device == '#')
+		return 0;	/* Ignore blank lines and comments */
 	mntpnt = parse_word(&cp);
 	type = parse_word(&cp);
 	opts = parse_word(&cp);
 	freq = parse_word(&cp);
 	passno = parse_word(&cp);
 
-	if (!device)
-		return 0;	/* Allow blank lines */
-	
 	if (!mntpnt || !type)
 		return -1;
 
@@ -867,6 +864,16 @@ static int ignore(struct fs_info *fs)
 	if (fs->passno == 0)
 		return 1;
 
+	/*
+	 * If this is a bind mount, ignore it.
+	 */
+	if (opt_in_list("bind", fs->opts)) {
+		fprintf(stderr,
+			_("%s: skipping bad line in /etc/fstab: bind mount with nonzero fsck pass number\n"),
+			fs->mountpt);
+		return 1;
+	}
+
 	interpret_type(fs);
 
 	/*
